@@ -48,6 +48,10 @@
 
 static CamelOfflineFolderClass *parent_class = NULL;
 
+static CamelProperty exchange_property_list[] = {
+	{ CAMEL_EXCHANGE_FOLDER_CHECK_FOLDER, "check_folder", N_("Always check for new mail in this folder") },
+};
+
 /* Returns the class for a CamelFolder */
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
@@ -84,6 +88,9 @@ static void   transfer_messages_the_hard_way (CamelFolder *source,
 static void refresh_info (CamelFolder *folder, CamelException *ex);
 static void exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
 
+static int exchange_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args);
+static int exchange_setv (CamelObject *object, CamelException *ex, CamelArgV *args);
+
 static void
 class_init (CamelFolderClass *camel_folder_class)
 {
@@ -100,6 +107,9 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 	camel_folder_class->refresh_info = refresh_info;
 	camel_folder_class->sync = exchange_sync;
+
+ 	((CamelObjectClass *)camel_folder_class)->getv = exchange_getv;
+	((CamelObjectClass *)camel_folder_class)->setv = exchange_setv;
 }
 
 #define CAMEL_EXCHANGE_SERVER_FLAGS \
@@ -141,6 +151,8 @@ camel_exchange_folder_get_type (void)
 	static CamelType camel_exchange_folder_type = CAMEL_INVALID_TYPE;
 
 	if (camel_exchange_folder_type == CAMEL_INVALID_TYPE) {
+		int i;
+
 		camel_exchange_folder_type = camel_type_register (
 			camel_offline_folder_get_type (),
 			"CamelExchangeFolder",
@@ -150,6 +162,10 @@ camel_exchange_folder_get_type (void)
 			NULL,
 			(CamelObjectInitFunc) init,
 			(CamelObjectFinalizeFunc) finalize );
+
+		/* only localize here, do not create GSList, we do not want to leak */
+		for (i = 0; i < sizeof (exchange_property_list)/sizeof (exchange_property_list [0]); i++)
+			exchange_property_list [i].description = _(exchange_property_list [i].description);
 	}
 
 	return camel_exchange_folder_type;
@@ -202,7 +218,7 @@ refresh_info (CamelFolder *folder, CamelException *ex)
 
 	if (camel_exchange_store_connected (store, ex)) {
 		camel_offline_journal_replay (exch->journal, NULL);
-	
+
 		camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_REFRESH_FOLDER,
 				 CAMEL_STUB_ARG_FOLDER, folder->full_name,
 				 CAMEL_STUB_ARG_END);
@@ -289,20 +305,20 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 	GString *new_subject;
 	int i, len;
 
-	/* 
-	   FIXME: We should add a top-level camel API camel_mime_message_prune_invalid_chars 
-	   which each of the provider will have to implement to remove things 
-	   that are invalid for their Transport mechanism. This will help in 
+	/*
+	   FIXME: We should add a top-level camel API camel_mime_message_prune_invalid_chars
+	   which each of the provider will have to implement to remove things
+	   that are invalid for their Transport mechanism. This will help in
 	   avoiding  duplication of work. Now Sending and Importing both requires
-	   substitution of \t and \n with blank. 
+	   substitution of \t and \n with blank.
 	*/
-	
+
 	old_subject = g_strdup(camel_mime_message_get_subject (message));
-	
+
 	if (old_subject) {
 		len = strlen (old_subject);
 		new_subject = g_string_new("");
-		for (i = 0; i < len; i++) 
+		for (i = 0; i < len; i++)
 			if ((old_subject[i] != '\t') && (old_subject[i] != '\n'))
 				new_subject = g_string_append_c (new_subject, old_subject[i]);
 			else
@@ -310,7 +326,7 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 		camel_mime_message_set_subject (message, new_subject->str);
 		g_free (old_subject);
 		g_string_free (new_subject, TRUE);
-	}	
+	}
 
 	if (!camel_exchange_store_connected (store, ex)) {
 		camel_exchange_journal_append ((CamelExchangeJournal *) ((CamelExchangeFolder *)folder)->journal, message, info, appended_uid, ex);
@@ -336,9 +352,9 @@ fix_broken_multipart_related (CamelMimePart *part)
 	CamelMultipart *multipart, *new;
 	CamelMimePart *subpart;
 	int i, count, broken_parts;
-	
+
 	content = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-	
+
 	content_type = content->mime_type;
 	if (camel_content_type_is (content_type, "message", "rfc822")) {
 		fix_broken_multipart_related (CAMEL_MIME_PART (content));
@@ -407,7 +423,7 @@ get_message_data (CamelFolder *folder, const char *uid, CamelException *ex)
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
                                      _("This message is not available in offline mode."));
 		return NULL;
-	}	
+	}
 
 	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_MESSAGE,
 			      CAMEL_STUB_ARG_FOLDER, folder->full_name,
@@ -475,7 +491,7 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 			}
 			g_strfreev (list_headers);
 		}
-		
+
 		if (!found_list)
 			camel_medium_set_header (CAMEL_MEDIUM (msg), "X-Evolution-Mail-From-Delegate", "yes");
 	}
@@ -669,19 +685,19 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	if (!camel_exchange_store_connected (store, ex)) {
 		CamelExchangeJournal *journal = (CamelExchangeJournal *) exch_dest->journal;
 		CamelMimeMessage *message;
-		
+
 		for (i = 0; i < uids->len; i++) {
 			info = camel_folder_summary_uid (source->summary, uids->pdata[i]);
 			if (!info)
 				continue;
-			
+
 			if (!(message = get_message (source, camel_message_info_uid (info), ex)))
 				break;
-			
-			camel_exchange_journal_transfer (journal, exch_source, message, 
+
+			camel_exchange_journal_transfer (journal, exch_source, message,
 							 info, uids->pdata[i], NULL,
 							 delete_originals, ex);
-							 
+
 			camel_object_unref (message);
 
 			if (camel_exception_is_set (ex))
@@ -949,7 +965,7 @@ camel_exchange_folder_update_message_flags_ex (CamelExchangeFolder *exch,
 	if (!mask) {
 		return;
 	}
-	
+
 	if ((info->flags & mask) != (flags & mask)) {
 		info->flags &= ~mask;
 		info->flags |= (flags & mask);
@@ -1132,10 +1148,6 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		if (!ok)
 			return FALSE;
 
-		if (len)
-			return TRUE;
-
-
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER)
 			folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER_JUNK)
@@ -1145,6 +1157,10 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 		if (offline_state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL )
 			return TRUE;
+		
+		if (len)
+			return TRUE;
+
 		camel_operation_start (NULL, _("Fetching summary information for new messages"));
 		ok = camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_REFRESH_FOLDER,
 				      CAMEL_STUB_ARG_FOLDER, folder->full_name,
@@ -1157,15 +1173,90 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 	if (camel_exchange_summary_get_readonly (folder->summary))
 		folder->permanent_flags = 0;
-	
+
 	return TRUE;
 }
 
-static void 
+static void
 exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 {
 	if (expunge)
 		exchange_expunge (folder, ex);
-	
+
 	camel_folder_summary_save (folder->summary);
 }
+
+static int
+exchange_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
+{
+	int i, count=0;
+	guint32 tag;
+
+	for (i = 0; i < args->argc; i++) {
+		CamelArgGet *arg = &args->argv[i];
+
+		tag = arg->tag;
+
+		switch (tag & CAMEL_ARG_TAG) {
+		case CAMEL_OBJECT_ARG_PERSISTENT_PROPERTIES:
+		case CAMEL_FOLDER_ARG_PROPERTIES: {
+			CamelArgGetV props;
+			int i;
+
+			props.argc = 1;
+			props.argv[0] = *arg;
+			((CamelObjectClass *)parent_class)->getv(object, ex, &props);
+
+			for (i = 0; i < sizeof (exchange_property_list)/sizeof (exchange_property_list [0]); i++)
+				*arg->ca_ptr = g_slist_append (*arg->ca_ptr, &exchange_property_list[i]);
+			break; }
+			/* exchange args */
+		case CAMEL_EXCHANGE_FOLDER_ARG_CHECK_FOLDER:
+			*arg->ca_int = ((CamelExchangeFolder *)object)->check_folder;
+			break;
+		default:
+			count++;
+			continue;
+		}
+
+		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
+	}
+
+	if (count)
+		return ((CamelObjectClass *)parent_class)->getv (object, ex, args);
+
+	return 0;
+}
+
+static int
+exchange_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
+{
+	int save = 0;
+	int i;
+	guint32 tag;
+
+	for (i = 0; i < args->argc; i++) {
+		CamelArg *arg = &args->argv[i];
+
+		tag = arg->tag;
+
+		switch (tag & CAMEL_ARG_TAG) {
+		case CAMEL_EXCHANGE_FOLDER_ARG_CHECK_FOLDER:
+			if (((CamelExchangeFolder *)object)->check_folder != arg->ca_int) {
+				((CamelExchangeFolder *)object)->check_folder = arg->ca_int;
+				save = 1;
+			}
+			break;
+		default:
+			continue;
+		}
+
+		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
+	}
+
+	if (save)
+		camel_object_state_write (object);
+
+	return ((CamelObjectClass *)parent_class)->setv (object, ex, args);
+}
+
